==> rumdl-0.0.137.tar.gz: rumdl-0.0.137/tests/lsp_formatting_tests.rs <==
//! Comprehensive tests for LSP document formatting capability
//! These tests ensure that the LSP server properly advertises and implements
//! document formatting, preventing regressions like issue #72

#![allow(deprecated)] // root_path is deprecated but required for InitializeParams

use tower_lsp::lsp_types::*;
use tower_lsp::{LanguageServer, LspService};

use rumdl_lib::lsp::RumdlLspConfig;
use rumdl_lib::lsp::server::RumdlLanguageServer;

/// Test that the server advertises document formatting capability
#[tokio::test]
async fn test_server_advertises_formatting_capability() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    let result = service.inner().initialize(init_params).await;
    assert!(result.is_ok(), "Initialization should succeed");

    let init_result = result.unwrap();
    let caps = init_result.capabilities;

    // CRITICAL: Verify that document formatting is advertised
    assert!(
        caps.document_formatting_provider.is_some(),
        "Server must advertise document_formatting_provider capability"
    );

    match caps.document_formatting_provider {
        Some(OneOf::Left(enabled)) => {
            assert!(enabled, "Document formatting should be enabled");
        }
        Some(OneOf::Right(_options)) => {
            // If using options, they're properly configured by definition
            // since we're receiving them from the server
        }
        None => {
            panic!("Document formatting provider must be advertised");
        }
    }
}

/// Test the complete formatting request/response flow
#[tokio::test]
async fn test_formatting_request_flow() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize first
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open a document with formatting issues
    let uri = Url::parse("file:///test/format.md").unwrap();
    let text = "# Heading\n\nText with trailing spaces   \n\nNo final newline";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri: uri.clone() },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    // Verify we get a valid response
    assert!(result.is_ok(), "Formatting request should succeed");

    let edits = result.unwrap();
    assert!(edits.is_some(), "Should return edits for document with issues");

    let edits = edits.unwrap();
    assert!(!edits.is_empty(), "Should have at least one edit");

    // Verify the edit fixes the issues
    let edit = &edits[0];
    assert!(
        edit.new_text.ends_with('\n'),
        "Formatted text should have final newline"
    );
    assert!(
        !edit.new_text.contains("   \n"),
        "Formatted text should not have trailing spaces"
    );
}

/// Test formatting with no fixable issues
#[tokio::test]
async fn test_formatting_no_issues() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open a perfect document
    let uri = Url::parse("file:///test/perfect.md").unwrap();
    let text = "# Perfect Document\n\nNo issues here.\n";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed even with no issues");
    let edits = result.unwrap();
    assert!(
        edits.as_ref().is_none_or(|e| e.is_empty()),
        "Should return None or empty edits for perfect document"
    );
}

/// Test formatting with multiple issues
#[tokio::test]
async fn test_formatting_multiple_issues() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open document with multiple formatting issues
    let uri = Url::parse("file:///test/multiple.md").unwrap();
    let text = "#Missing space\n\nTrailing spaces   \n-  Wrong list spacing\n1.  Wrong ordered list";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed");
    let edits = result.unwrap();
    assert!(edits.is_some(), "Should return edits for document with multiple issues");

    let edits = edits.unwrap();
    assert!(!edits.is_empty(), "Should have edits");

    // Verify the formatted text addresses the issues
    let edit = &edits[0];
    assert!(
        edit.new_text.starts_with("# Missing space"),
        "Should fix missing space after heading marker"
    );
    assert!(!edit.new_text.contains("   \n"), "Should remove trailing spaces");
    assert!(
        edit.new_text.contains("- Wrong list spacing") || edit.new_text.contains("-  Wrong list spacing"),
        "Should handle list spacing"
    );
}

/// Test formatting respects rumdl config disabled rules
#[tokio::test]
async fn test_formatting_respects_config_disabled_rules() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize with a config path that would have disabled rules
    // Note: In a real scenario, the rumdl config would be loaded from a file
    // that specifies disabled rules. Since we can't easily mock that here,
    // this test verifies the mechanism is in place.
    let custom_config = RumdlLspConfig {
        config_path: None,
        enable_linting: true,
        enable_auto_fix: true,
        enable_rules: None,
        disable_rules: None,
    };

    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: Some(serde_json::to_value(custom_config).unwrap()),
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open document with trailing spaces
    let uri = Url::parse("file:///test/config_test.md").unwrap();
    let text = "# Heading\n\nText with trailing spaces   \n";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed");
    let edits = result.unwrap();

    // The formatting now respects the rumdl configuration's disabled rules
    // If MD009 were disabled in the rumdl config, it wouldn't fix trailing spaces
    // Since we're using default config here, MD009 is enabled and will fix trailing spaces
    if let Some(edits) = edits.as_ref()
        && let Some(edit) = edits.first()
    {
        assert!(
            !edit.new_text.contains("   \n"),
            "Formatting should fix trailing spaces when MD009 is not disabled in rumdl config"
        );
    }
}

/// Test formatting for empty document
#[tokio::test]
async fn test_formatting_empty_document() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open empty document
    let uri = Url::parse("file:///test/empty.md").unwrap();
    let text = "";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting empty document should not error");
    let edits = result.unwrap();
    assert!(
        edits.as_ref().is_none_or(|e| e.is_empty()),
        "Empty document should have no formatting edits"
    );
}

/// Test that formatting preserves document structure
#[tokio::test]
async fn test_formatting_preserves_structure() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open document with complex structure
    let uri = Url::parse("file:///test/structure.md").unwrap();
    let text = r#"# Main Title

## Section 1

Some text here.

### Subsection

- List item 1
- List item 2

```python
def hello():
    print("world")
```

## Section 2

> Quote with trailing spaces

Final paragraph"#;

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed");
    let edits = result.unwrap();

    if let Some(edits) = edits.as_ref()
        && let Some(edit) = edits.first()
    {
        // Verify structure is preserved
        assert!(edit.new_text.contains("# Main Title"), "Main title preserved");
        assert!(edit.new_text.contains("## Section 1"), "Section headers preserved");
        assert!(edit.new_text.contains("```python"), "Code blocks preserved");
        assert!(edit.new_text.contains("def hello():"), "Code content preserved");
        assert!(edit.new_text.contains("> Quote"), "Blockquotes preserved");
        assert!(edit.new_text.contains("- List item"), "Lists preserved");

        // MD009 doesn't fix trailing spaces in list items with just the spaces
        // It only fixes when there's "   \n" pattern. List items ending with just spaces
        // are not fixed. This is the actual behavior.
        // For now, just verify the structure is preserved

        // Verify final newline added
        assert!(edit.new_text.ends_with('\n'), "Final newline added");
    }
}

==> rumdl-0.0.146.tar.gz: rumdl-0.0.146/tests/lsp_formatting_tests.rs <==
//! Comprehensive tests for LSP document formatting capability
//! These tests ensure that the LSP server properly advertises and implements
//! document formatting, preventing regressions like issue #72

#![allow(deprecated)] // root_path is deprecated but required for InitializeParams

use tower_lsp::lsp_types::*;
use tower_lsp::{LanguageServer, LspService};

use rumdl_lib::lsp::RumdlLspConfig;
use rumdl_lib::lsp::server::RumdlLanguageServer;

/// Test that the server advertises document formatting capability
#[tokio::test]
async fn test_server_advertises_formatting_capability() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    let result = service.inner().initialize(init_params).await;
    assert!(result.is_ok(), "Initialization should succeed");

    let init_result = result.unwrap();
    let caps = init_result.capabilities;

    // CRITICAL: Verify that document formatting is advertised
    assert!(
        caps.document_formatting_provider.is_some(),
        "Server must advertise document_formatting_provider capability"
    );

    match caps.document_formatting_provider {
        Some(OneOf::Left(enabled)) => {
            assert!(enabled, "Document formatting should be enabled");
        }
        Some(OneOf::Right(_options)) => {
            // If using options, they're properly configured by definition
            // since we're receiving them from the server
        }
        None => {
            panic!("Document formatting provider must be advertised");
        }
    }
}

/// Test the complete formatting request/response flow
#[tokio::test]
async fn test_formatting_request_flow() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize first
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open a document with formatting issues
    let uri = Url::parse("file:///test/format.md").unwrap();
    let text = "# Heading\n\nText with trailing spaces   \n\nNo final newline";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri: uri.clone() },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    // Verify we get a valid response
    assert!(result.is_ok(), "Formatting request should succeed");

    let edits = result.unwrap();
    assert!(edits.is_some(), "Should return edits for document with issues");

    let edits = edits.unwrap();
    assert!(!edits.is_empty(), "Should have at least one edit");

    // Verify the edit fixes the issues
    let edit = &edits[0];
    assert!(
        edit.new_text.ends_with('\n'),
        "Formatted text should have final newline"
    );
    assert!(
        !edit.new_text.contains("   \n"),
        "Formatted text should not have trailing spaces"
    );
}

/// Test formatting with no fixable issues
#[tokio::test]
async fn test_formatting_no_issues() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open a perfect document
    let uri = Url::parse("file:///test/perfect.md").unwrap();
    let text = "# Perfect Document\n\nNo issues here.\n";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed even with no issues");
    let edits = result.unwrap();
    assert!(
        edits.as_ref().is_some_and(|e| e.is_empty()),
        "Should return empty edits array for perfect document"
    );
}

/// Test formatting with multiple issues
#[tokio::test]
async fn test_formatting_multiple_issues() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open document with multiple formatting issues
    let uri = Url::parse("file:///test/multiple.md").unwrap();
    let text = "#Missing space\n\nTrailing spaces   \n-  Wrong list spacing\n1.  Wrong ordered list";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed");
    let edits = result.unwrap();
    assert!(edits.is_some(), "Should return edits for document with multiple issues");

    let edits = edits.unwrap();
    assert!(!edits.is_empty(), "Should have edits");

    // Verify the formatted text addresses the issues
    let edit = &edits[0];
    assert!(
        edit.new_text.starts_with("# Missing space"),
        "Should fix missing space after heading marker"
    );
    assert!(!edit.new_text.contains("   \n"), "Should remove trailing spaces");
    assert!(
        edit.new_text.contains("- Wrong list spacing") || edit.new_text.contains("-  Wrong list spacing"),
        "Should handle list spacing"
    );
}

/// Test formatting respects rumdl config disabled rules
#[tokio::test]
async fn test_formatting_respects_config_disabled_rules() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize with a config path that would have disabled rules
    // Note: In a real scenario, the rumdl config would be loaded from a file
    // that specifies disabled rules. Since we can't easily mock that here,
    // this test verifies the mechanism is in place.
    let custom_config = RumdlLspConfig {
        config_path: None,
        enable_linting: true,
        enable_auto_fix: true,
        enable_rules: None,
        disable_rules: None,
    };

    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: Some(serde_json::to_value(custom_config).unwrap()),
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open document with trailing spaces
    let uri = Url::parse("file:///test/config_test.md").unwrap();
    let text = "# Heading\n\nText with trailing spaces   \n";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed");
    let edits = result.unwrap();

    // The formatting now respects the rumdl configuration's disabled rules
    // If MD009 were disabled in the rumdl config, it wouldn't fix trailing spaces
    // Since we're using default config here, MD009 is enabled and will fix trailing spaces
    if let Some(edits) = edits.as_ref()
        && let Some(edit) = edits.first()
    {
        assert!(
            !edit.new_text.contains("   \n"),
            "Formatting should fix trailing spaces when MD009 is not disabled in rumdl config"
        );
    }
}

/// Test formatting for empty document
#[tokio::test]
async fn test_formatting_empty_document() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open empty document
    let uri = Url::parse("file:///test/empty.md").unwrap();
    let text = "";

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting empty document should not error");
    let edits = result.unwrap();
    assert!(
        edits.as_ref().is_some_and(|e| e.is_empty()),
        "Empty document should return empty edits array"
    );
}

/// Test that formatting preserves document structure
#[tokio::test]
async fn test_formatting_preserves_structure() {
    let (service, _socket) = LspService::new(RumdlLanguageServer::new);

    // Initialize
    let init_params = InitializeParams {
        process_id: None,
        root_path: None, // Deprecated but required
        root_uri: Some(Url::parse("file:///test").unwrap()),
        initialization_options: None,
        capabilities: ClientCapabilities::default(),
        trace: None,
        workspace_folders: None,
        client_info: None,
        locale: None,
    };

    service.inner().initialize(init_params).await.unwrap();
    service.inner().initialized(InitializedParams {}).await;

    // Open document with complex structure
    let uri = Url::parse("file:///test/structure.md").unwrap();
    let text = r#"# Main Title

## Section 1

Some text here.

### Subsection

- List item 1
- List item 2

```python
def hello():
    print("world")
```

## Section 2

> Quote with trailing spaces

Final paragraph"#;

    service
        .inner()
        .did_open(DidOpenTextDocumentParams {
            text_document: TextDocumentItem {
                uri: uri.clone(),
                language_id: "markdown".to_string(),
                version: 1,
                text: text.to_string(),
            },
        })
        .await;

    // Request formatting
    let formatting_params = DocumentFormattingParams {
        text_document: TextDocumentIdentifier { uri },
        options: FormattingOptions {
            tab_size: 4,
            insert_spaces: true,
            properties: std::collections::HashMap::new(),
            trim_trailing_whitespace: Some(true),
            insert_final_newline: Some(true),
            trim_final_newlines: Some(true),
        },
        work_done_progress_params: WorkDoneProgressParams::default(),
    };

    let result = service.inner().formatting(formatting_params).await;

    assert!(result.is_ok(), "Formatting should succeed");
    let edits = result.unwrap();

    if let Some(edits) = edits.as_ref()
        && let Some(edit) = edits.first()
    {
        // Verify structure is preserved
        assert!(edit.new_text.contains("# Main Title"), "Main title preserved");
        assert!(edit.new_text.contains("## Section 1"), "Section headers preserved");
        assert!(edit.new_text.contains("```python"), "Code blocks preserved");
        assert!(edit.new_text.contains("def hello():"), "Code content preserved");
        assert!(edit.new_text.contains("> Quote"), "Blockquotes preserved");
        assert!(edit.new_text.contains("- List item"), "Lists preserved");

        // MD009 doesn't fix trailing spaces in list items with just the spaces
        // It only fixes when there's "   \n" pattern. List items ending with just spaces
        // are not fixed. This is the actual behavior.
        // For now, just verify the structure is preserved

        // Verify final newline added
        assert!(edit.new_text.ends_with('\n'), "Final newline added");
    }
}
